<?php
/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://club.wpeka.com/
 * @since      1.0.0
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/admin
 * @author     WPEka Club <support@wpeka.com>
 */
class Wpadcenter_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string $plugin_name       The name of this plugin.
	 * @param      string $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wpadcenter_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wpadcenter_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_register_style( $this->plugin_name . '-settings', plugin_dir_url( __FILE__ ) . 'css/wpadcenter-admin-settings' . WPADCENTER_SCRIPT_SUFFIX . '.css', array(), $this->version, 'all' );
		wp_register_style( $this->plugin_name . '-getting-started', plugin_dir_url( __FILE__ ) . 'css/wpadcenter-admin-getting-started' . WPADCENTER_SCRIPT_SUFFIX . '.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wpadcenter_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wpadcenter_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$wpadcenter_cpts = array( 'wpadcenter-adzones', 'wpadcenter-adverts', 'wpadcenter-campaigns', 'wpadcenter-banners' );
		if ( in_array( get_post_type(), $wpadcenter_cpts, true ) ) {
			wp_dequeue_script( 'autosave' );
		}

		wp_register_script( $this->plugin_name . '-settings', plugin_dir_url( __FILE__ ) . 'js/wpadcenter-admin-settings' . WPADCENTER_SCRIPT_SUFFIX . '.js', array( 'jquery' ), $this->version, false );
	}

	/**
	 * Register the JavaScript for the ACF fields.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_enqueue_acf_scripts() {
		wp_enqueue_script( $this->plugin_name . '-acf', plugin_dir_url( __FILE__ ) . 'js/wpadcenter-admin-acf' . WPADCENTER_SCRIPT_SUFFIX . '.js', array( 'jquery' ), $this->version, false );
	}

	/**
	 * Adds action links to the plugin list table.
	 *
	 * Fired by `plugin_action_links` filter.
	 *
	 * @since 1.0.0
	 *
	 * @param array $links An array of plugin action links.
	 * @return array An array of plugin action links.
	 */
	public function wpadcenter_plugin_action_links( $links ) {
		if ( ! get_option( 'wpadcenter_pro_installed' ) ) {
			$links['go_pro'] = sprintf( '<a href="%1$s" target="_blank" class="wpadcenter-gopro">%2$s</a>', 'https://club.wpeka.com/latest-plugins/wpadcenter/', __( 'Go Pro', 'wpadcenter' ) );
		}
		return $links;
	}

	/**
	 * Register the menu for the plugin.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_admin_menu() {
		// Create Menu for WP AdCenter.
		add_menu_page( 'WP AdCenter', 'WP AdCenter', 'manage_options', 'wpadcenter-settings', array( $this, 'wpadcenter_settings' ), WPADCENTER_PLUGIN_URL . 'images/menu-icon.png' );
		add_submenu_page( 'wpadcenter-settings', 'Settings', __( 'Settings', 'wpadcenter' ), 'manage_options', 'wpadcenter-settings', array( $this, 'wpadcenter_settings' ) );
		add_submenu_page( 'wpadcenter-settings', 'Ad Zones', __( 'Ad Zones', 'wpadcenter' ), 'manage_options', 'edit.php?post_type=wpadcenter-adzones' );
		add_submenu_page( 'wpadcenter-settings', 'Advertisers', __( 'Advertisers', 'wpadcenter' ), 'manage_options', 'edit.php?post_type=wpadcenter-adverts' );
		add_submenu_page( 'wpadcenter-settings', 'Campaigns', __( 'Campaigns', 'wpadcenter' ), 'manage_options', 'edit.php?post_type=wpadcenter-campaigns' );
		add_submenu_page( 'wpadcenter-settings', 'Banners', __( 'Banners', 'wpadcenter' ), 'manage_options', 'edit.php?post_type=wpadcenter-banners' );
		do_action( 'wpadcenter_admin_menu', 'wpadcenter-settings', 'manage_options' );
		add_submenu_page( 'wpadcenter-settings', 'Getting Started', __( 'Getting Started', 'wpadcenter' ), 'manage_options', 'wpadcenter-getting-started', array( $this, 'wpadcenter_getting_started' ) );
	}

	/**
	 * Register WP AdCenter widget.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_widget() {
		register_widget( 'Wpadcenter_Widget' );
	}

	/**
	 * Callback function for Settings menu.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_settings() {
		// Lock out non-admins.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		wp_enqueue_style( $this->plugin_name . '-settings' );
		wp_enqueue_script( $this->plugin_name . '-settings' );
		$the_options = Wpadcenter::wpadcenter_get_settings();
		// Check if form has been set.
		if ( isset( $_POST['update_admin_settings_form'] ) || ( isset( $_POST['wpadcenter_settings_ajax_update'] ) ) ) {
			// Check nonce.
			check_admin_referer( 'wpadcenter-update-' . WPADCENTER_SETTINGS_FIELD );
			if ( 'update_admin_settings_form' === $_POST['wpadcenter_settings_ajax_update'] ) {
				foreach ( $the_options as $key => $value ) {
					if ( isset( $_POST[ $key . '_field' ] ) ) {
						// Store sanitised values only.
						$the_options[ $key ] = Wpadcenter::wpadcenter_sanitise_settings( $key, wp_unslash( $_POST[ $key . '_field' ] ) ); // phpcs:ignore input var ok, CSRF ok, sanitization ok.
					}
				}
				$the_options = apply_filters( 'wpadcenter_after_save_settings', $the_options );
				update_option( WPADCENTER_SETTINGS_FIELD, $the_options );
				echo '<div class="updated"><p><strong>' . esc_attr__( 'Settings Updated.', 'wpadcenter' ) . '</strong></p></div>';
			}
		}
		require_once plugin_dir_path( __FILE__ ) . 'partials/wpadcenter-admin-display.php';
	}

	/**
	 * Callback function for Getting Started menu.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_getting_started() {
		// Lock out non-admins.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( esc_attr__( 'You do not have sufficient permission to perform this operation', 'wpadcenter' ) );
		}
		wp_enqueue_style( $this->plugin_name . '-getting-started' );
		require_once plugin_dir_path( __FILE__ ) . 'views/admin-display-getting-started.php';
	}

	/**
	 * Define CPT's.
	 *
	 * @since 1.0.0
	 * @return mixed|void
	 */
	public function wpadcenter_get_cpt_args() {
		$cpt_args              = array();
		$cpt_args['adzones']   = apply_filters(
			'wpadcenter_cpt_args_adzones',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_adzones',
					array(
						'name'               => __( 'Ad Zones', 'wpadcenter' ),
						'singular_name'      => __( 'Ad Zone', 'wpadcenter' ),
						'menu_name'          => __( 'Ad Zones', 'wpadcenter' ),
						'all_items'          => __( 'All Ad Zones', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Ad Zone', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Ad Zone', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Ad Zone', 'wpadcenter' ),
						'update_item'        => __( 'Update Ad Zone', 'wpadcenter' ),
						'view_item'          => __( 'View Ad Zone', 'wpadcenter' ),
						'view_items'         => __( 'View Ad Zones', 'wpadcenter' ),
						'search_items'       => __( 'Search Ad Zone', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-adzones' ),
				'capability_type'     => 'post',
			)
		);
		$cpt_args['adverts']   = apply_filters(
			'wpadcenter_cpt_args_adverts',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_adverts',
					array(
						'name'               => __( 'Advertisers', 'wpadcenter' ),
						'singular_name'      => __( 'Advertiser', 'wpadcenter' ),
						'menu_name'          => __( 'Advertisers', 'wpadcenter' ),
						'all_items'          => __( 'All Advertisers', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Advertiser', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Advertiser', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Advertiser', 'wpadcenter' ),
						'update_item'        => __( 'Update Advertiser', 'wpadcenter' ),
						'view_item'          => __( 'View Advertiser', 'wpadcenter' ),
						'view_items'         => __( 'View Advertisers', 'wpadcenter' ),
						'search_items'       => __( 'Search Advertiser', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-adverts' ),
				'capability_type'     => 'post',
			)
		);
		$cpt_args['campaigns'] = apply_filters(
			'wpadcenter_cpt_args_campaigns',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_campaigns',
					array(
						'name'               => __( 'Campaigns', 'wpadcenter' ),
						'singular_name'      => __( 'Campaign', 'wpadcenter' ),
						'menu_name'          => __( 'Campaigns', 'wpadcenter' ),
						'all_items'          => __( 'All Campaigns', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Campaign', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Campaign', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Campaign', 'wpadcenter' ),
						'update_item'        => __( 'Update Campaign', 'wpadcenter' ),
						'view_item'          => __( 'View Campaign', 'wpadcenter' ),
						'view_items'         => __( 'View Campaigns', 'wpadcenter' ),
						'search_items'       => __( 'Search Campaign', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-campaigns' ),
				'capability_type'     => 'post',
			)
		);
		$cpt_args['banners']   = apply_filters(
			'wpadcenter_cpt_args_banners',
			array(
				'labels'              => apply_filters(
					'wpadcenter_cpt_args_labels_banners',
					array(
						'name'               => __( 'Banners', 'wpadcenter' ),
						'singular_name'      => __( 'Banner', 'wpadcenter' ),
						'menu_name'          => __( 'Banners', 'wpadcenter' ),
						'all_items'          => __( 'All Banners', 'wpadcenter' ),
						'add_new_item'       => __( 'Add New Banner', 'wpadcenter' ),
						'add_new'            => __( 'Add New', 'wpadcenter' ),
						'new_item'           => __( 'New Banner', 'wpadcenter' ),
						'edit_item'          => __( 'Edit Banner', 'wpadcenter' ),
						'update_item'        => __( 'Update Banner', 'wpadcenter' ),
						'view_item'          => __( 'View Banner', 'wpadcenter' ),
						'view_items'         => __( 'View Banners', 'wpadcenter' ),
						'search_items'       => __( 'Search Banner', 'wpadcenter' ),
						'not_found'          => __( 'Not found', 'wpadcenter' ),
						'not_found_in_trash' => __( 'Not found in Trash', 'wpadcenter' ),
					)
				),
				'supports'            => array( 'title' ),
				'public'              => true,
				'show_ui'             => true,
				'show_in_menu'        => false,
				'show_in_admin_bar'   => false,
				'show_in_nav_menus'   => false,
				'can_export'          => true,
				'has_archive'         => false,
				'hierarchical'        => false,
				'exclude_from_search' => true,
				'show_in_rest'        => false,
				'publicly_queryable'  => false,
				'rewrite'             => array( 'slug' => 'wpadcenter-banners' ),
				'capability_type'     => 'post',
			)
		);
		return apply_filters( 'wpadcenter_cpt_args', $cpt_args );
	}

	/**
	 * Register CPT's.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_cpt() {
		$wpadcenter_cpt_args = $this->wpadcenter_get_cpt_args();
		foreach ( $wpadcenter_cpt_args as $key => $cpt_args ) {
			if ( ! post_type_exists( 'wpadcenter-' . $key ) ) {
				register_post_type( 'wpadcenter-' . $key, $cpt_args );
			}
		}
	}

	/**
	 * Register custom statuses for CPT - Adverts.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_cpt_adverts_status() {
		$adverts_statuses = apply_filters(
			'wpadcenter_adverts_statuses',
			array(
				'adc-approved' => array(
					'label'                     => _x( 'Approved', 'Advertiser Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Approved <span class="count">(%s)</span>', 'Approved <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-pending'  => array(
					'label'                     => _x( 'Pending', 'Advertiser Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Pending <span class="count">(%s)</span>', 'Pending <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-declined' => array(
					'label'                     => _x( 'Declined', 'Advertiser Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Declined <span class="count">(%s)</span>', 'Declined <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
			)
		);

		foreach ( $adverts_statuses as $adverts_status => $values ) {
			register_post_status( $adverts_status, $values );
		}
	}

	/**
	 * Register custom statuses for CPT - Campaigns.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_cpt_campaigns_status() {
		$campaigns_statuses = apply_filters(
			'wpadcenter_campaigns_statuses',
			array(
				'adc-paused'    => array(
					'label'                     => _x( 'Paused', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Paused <span class="count">(%s)</span>', 'Paused <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-running'   => array(
					'label'                     => _x( 'Running', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Running <span class="count">(%s)</span>', 'Running <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-scheduled' => array(
					'label'                     => _x( 'Scheduled', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Scheduled <span class="count">(%s)</span>', 'Scheduled <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
				'adc-finished'  => array(
					'label'                     => _x( 'Finished', 'Campaign Status', 'wpadcenter' ),
					'public'                    => true,
					/* translators: %s: number of advertisers */
					'label_count'               => _n_noop( 'Finished <span class="count">(%s)</span>', 'Finished <span class="count">(%s)</span>', 'wpadcenter' ),
					'show_in_admin_all_list'    => true,
					'show_in_admin_status_list' => true,
					'show_in_metabox_dropdown'  => true,
					'show_in_inline_dropdown'   => true,
				),
			)
		);

		foreach ( $campaigns_statuses as $campaign_status => $values ) {
			register_post_status( $campaign_status, $values );
		}
	}

	/**
	 * Register block.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_register_block_type() {
		if ( ! function_exists( 'register_block_type' ) ) {
			return;
		}
		$ad_zones      = array();
		$zone['label'] = __( 'Select Ad Zone', 'wpadcenter' );
		$zone['value'] = '0';
		$ad_zones[]    = $zone;
		$args          = array(
			'post_type'   => 'wpadcenter-adzones',
			'post_status' => 'publish',
			'numberposts' => -1,
		);
		$adzones       = get_posts( $args );
		if ( $adzones ) {
			foreach ( $adzones as $adzone ) {
				$zone['value'] = $adzone->ID;
				$zone['label'] = $adzone->post_title;
				$ad_zones[]    = $zone;
			}
		}
		wp_enqueue_style(
			$this->plugin_name . '-block',
			plugin_dir_url( __FILE__ ) . 'css/wpadcenter-admin-block' . WPADCENTER_SCRIPT_SUFFIX . '.css',
			array(),
			$this->version,
			'all'
		);
		wp_enqueue_script(
			$this->plugin_name . '-cycle2',
			plugin_dir_url( __FILE__ ) . 'js/libraries/cycle2/cycle2.js',
			array(
				'jquery',
			),
			$this->version,
			true
		);
		wp_enqueue_script(
			$this->plugin_name . '-block',
			plugin_dir_url( __FILE__ ) . 'js/blocks/wpadcenter-admin-block.js',
			array(
				'jquery',
				'wp-blocks',
				'wp-i18n',
				'wp-editor',
				'wp-element',
				'wp-components',
			),
			$this->version,
			false
		);
		wp_localize_script( $this->plugin_name . '-block', 'ad_zones', $ad_zones );
		register_block_type(
			'wpadcenter/block',
			array(
				'editor_script'   => $this->plugin_name . '-block',
				'render_callback' => array( $this, 'wpadcenter_block_render_callback' ),
				'attributes'      => [
					'id' => [
						'default' => '0',
						'type'    => 'string,',
					],
				],
			)
		);
	}

	/**
	 * Render callback for block.
	 *
	 * @since 1.0.0
	 *
	 * @param Array $atts Shortcode attributes.
	 * @return string
	 */
	public function wpadcenter_block_render_callback( $atts ) {
		$container_styles = '';
		$message          = '';
		if ( isset( $atts['id'] ) && '0' !== $atts['id'] ) {
			$adzone_id    = sanitize_text_field( wp_unslash( $atts['id'] ) );
			$the_options  = Wpadcenter::wpadcenter_get_settings();
			$geo_location = $the_options['geo_location'];
			$auto_refresh = $the_options['auto_refresh'];
			if ( $auto_refresh ) {
				$transition = $the_options['transition_effect'];
				$speed      = $the_options['transition_speed'];
				$delay      = $the_options['transition_delay'];
			}
			$numberofposts     = -1;
			$container_styles .= 'width:100%;height:auto;margin:0;padding:0;';
			$adzone_display    = false;
			if ( 'adzone' === $geo_location && get_option( 'wpadcenter_pro_active' ) ) {
				$adzone_geo_countries = get_field( 'adzones_geolocation_countries', $adzone_id );
				if ( $adzone_geo_countries ) {
					$geo_country = Wpadcenter::wpadcenter_get_geo_country_code();
					if ( in_array( $geo_country, $adzone_geo_countries, true ) || in_array( 'all', $adzone_geo_countries, true ) ) {
						$adzone_display = true;
					}
				} else {
					$adzone_display = true;
				}
			} else {
				$adzone_display = true;
			}
			if ( $adzone_display ) {
				if ( ! $auto_refresh ) {
					$auto_refresh = get_field( 'auto_refresh_ads', $adzone_id );
					$transition   = get_field( 'transition_effect', $adzone_id );
					$speed        = get_field( 'transition_speed', $adzone_id );
					$delay        = get_field( 'transition_delay', $adzone_id );
				}
				$adzone_default_banner = get_field( 'default_banner', $adzone_id );
				$adzone_signup_link    = get_field( 'signup_link', $adzone_id );
				$adzone_template       = get_field( 'template', $adzone_id );
				$adzone_text_above     = get_field( 'text_above_ad_zone', $adzone_id );
				$adzone_text_below     = get_field( 'text_below_ad_zone', $adzone_id );
				$adzone_size           = explode( 'x', $adzone_template );
				$width                 = $adzone_size['0'];
				$height                = $adzone_size['1'];
				$adzone_rows           = get_field( 'horizontal', $adzone_id );
				$adzone_columns        = get_field( 'vertical', $adzone_id );
				$adzone_classes        = 'adzone_1_1';
				if ( $adzone_rows && $adzone_columns && ! $auto_refresh ) {
					$adzone_classes = 'adzone_' . $adzone_rows . '_' . $adzone_columns;
					$numberofposts  = $adzone_columns * $adzone_rows;
				}
				$args      = array(
					'post_type'   => 'wpadcenter-campaigns',
					'post_status' => 'adc-running',
					'numberposts' => -1,
				);
				$campaigns = get_posts( $args );
				if ( $campaigns ) {
					$campaign_ids = array();
					foreach ( $campaigns as $campaign ) {
						$campaign_id = $campaign->ID;
						if ( 'campaign' === $geo_location && get_option( 'wpadcenter_pro_active' ) ) {
							$campaign_geo_countries = get_field( 'campaigns_geolocation_countries', $campaign->ID );
							if ( $campaign_geo_countries ) {
								$geo_country = Wpadcenter::wpadcenter_get_geo_country_code();
								if ( in_array( $geo_country, $campaign_geo_countries, true ) || in_array( 'all', $campaign_geo_countries, true ) ) {
									$campaign_ids[] = $campaign_id;
								}
							} else {
								$campaign_ids[] = $campaign_id;
							}
						} else {
							$campaign_ids[] = $campaign_id;
						}
					}
					if ( empty( $campaign_ids ) ) {
						$campaign_ids = '';
					}
					$args    = array(
						'post_type'   => 'wpadcenter-banners',
						'post_status' => 'publish',
						'numberposts' => $numberofposts,
						'meta_query'  => array( // phpcs:ignore slow query
							'relation' => 'AND',
							array(
								'key'     => 'banners_campaign',
								'value'   => $campaign_ids,
								'compare' => 'IN',
							),
							array(
								'key'     => 'banners_ad_zone',
								'value'   => $adzone_id,
								'compare' => '=',
							),
						),
					);
					$banners = get_posts( $args );
					if ( $banners ) {
						$message  = apply_filters( 'wpadcenter_block_render_before', $message );
						$message .= '<div class="wpadcenter-text-above">' . $adzone_text_above . '</div>';
						if ( $auto_refresh ) {
							$container_styles .= 'max-width:' . $width . 'px;';
							$message          .= '<div class="wpadcenter-slider cycle-slideshow" data-slides="div.wpadcenter-slide" data-cycle-fx="' . $transition . '" data-cycle-speed="' . $speed . '" data-cycle-delay="' . $delay . '">';
						} else {
							$message .= '<div class="wpadcenter-slider">';
						}
						foreach ( $banners as $banner ) {
							$banner_id          = $banner->ID;
							$banner_link_url    = get_field( 'link_url', $banner_id );
							$banner_link_target = get_field( 'link_target', $banner_id );
							$banner_advertiser  = get_field( 'banners_advertiser', $banner_id );
							$banner_campaign    = get_field( 'banners_campaign', $banner_id );
							$banner_ad_zone     = get_field( 'banners_ad_zone', $banner_id );
							$banner_type        = get_field( 'banner_type', $banner_id );
							$message           .= '<div class="wpadcenter-slide ' . $adzone_classes . '">';
							if ( ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) && get_option( 'wpadcenter_pro_active' ) ) {
								$message .= '<input type="hidden" name="banner_id" value="' . $banner_id . '">';
								$message .= '<input type="hidden" name="banner_advertiser" value="' . $banner_advertiser . '">';
								$message .= '<input type="hidden" name="banner_campaign" value="' . $banner_campaign . '">';
								$message .= '<input type="hidden" name="banner_ad_zone" value="' . $banner_ad_zone . '">';
							}
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '<a href="' . $banner_link_url . '" target="' . $banner_link_target . '">';
							}
							switch ( $banner_type ) {
								case 'image':
									$banner_img = get_field( 'image', $banner_id );
									$banner_img = wp_get_attachment_image( $banner_img, $width, $height );
									$message   .= $banner_img;
									break;
								case 'external_image_link':
									$banner_img_link = get_field( 'image_link', $banner_id );
									$message        .= '<img width="' . $width . '" height="' . $height . '" src="' . esc_url( $banner_img_link ) . '"/>';
									break;
								case 'ad_code':
									$banner_ad_code = get_field( 'ad_code', $banner_id );
									$message       .= '<span class="wpadcenter-ad-code">' . $banner_ad_code . '</span>';
									break;
							}
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '</a>';
							}
							$message .= '</div>';
						}
						$message .= '</div>';
						$message .= '<div class="wpadcenter-text-below">' . $adzone_text_below . '</div>';
					} else {
						if ( $adzone_default_banner ) {
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '<a href="' . $adzone_signup_link . '" target="_blank">';
							}
							$default_banner_img = wp_get_attachment_image( $adzone_default_banner, $width, $height );
							$message           .= $default_banner_img;
							if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
								$message .= '</a>';
							}
						} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
							$message .= __( 'No Ads found.', 'wpadcenter' );
						}
					}
				} else {
					if ( $adzone_default_banner ) {
						if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
							$message .= '<a href="' . $adzone_signup_link . '" target="_blank">';
						}
						$default_banner_img = wp_get_attachment_image( $adzone_default_banner, $width, $height );
						$message           .= $default_banner_img;
						if ( ! defined( 'REST_REQUEST' ) || ! REST_REQUEST ) {
							$message .= '</a>';
						}
					} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
						$message .= __( 'No Campaigns found.', 'wpadcenter' );
					}
				}
			} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
				$message .= __( 'No Ads found.', 'wpadcenter' );
			}
		} elseif ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
			$message .= __( 'Please choose ad zone.', 'wpadcenter' );
		}
		$message = apply_filters( 'wpadcenter_block_render_after', $message );
		return '<div class="wpadcenter-block-container" style="' . $container_styles . '">' . $message . '</div>';
	}

	/**
	 * Add Custom fields for CPT's - Ad Zones, Adverts, Campaigns, Banners.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_add_acf_fields() {

		$the_options = Wpadcenter::wpadcenter_get_settings();

		do_action( 'wpadcenter_before_adzones_acf_fields' );

		if ( function_exists( 'acf_add_local_field_group' ) ) {
			// Ad Zones – General.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_adzones_content',
					'title'                 => 'Content',
					'fields'                => apply_filters(
						'wpadcenter_adzones_content_fields',
						array(
							array(
								'key'               => 'field_adzones_description',
								'label'             => 'Description',
								'name'              => 'adzones_description',
								'type'              => 'textarea',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
								'maxlength'         => '',
								'rows'              => '',
								'new_lines'         => '',
							),
							array(
								'key'               => 'field_text_above_ad_zone',
								'label'             => 'Text above Ad Zone',
								'name'              => 'text_above_ad_zone',
								'type'              => 'textarea',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
								'maxlength'         => '',
								'rows'              => '',
								'new_lines'         => '',
							),
							array(
								'key'               => 'field_text_below_ad_zone',
								'label'             => 'Text below Ad Zone',
								'name'              => 'text_below_ad_zone',
								'type'              => 'textarea',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
								'maxlength'         => '',
								'rows'              => '',
								'new_lines'         => '',
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-adzones',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'acf_after_title',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);

			// Ad Zones – Settings.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_adzones_general',
					'title'                 => 'General Settings',
					'fields'                => apply_filters(
						'wpadcenter_adzones_general_fields',
						array(
							array(
								'key'               => 'field_template',
								'label'             => 'Template',
								'name'              => 'template',
								'type'              => 'select',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'choices'           => apply_filters(
									'wpadcenter_adzones_template_field_choices',
									array(
										'468x60'  => 'IAB Full Banner (468 x 60)',
										'120x600' => 'IAB Skyscraper (120 x 600)',
										'160x600' => 'IAB Wide Skyscraper (160 x 600)',
										'728x90'  => 'IAB Leaderboard (728 x 90)',
										'180x150' => 'IAB Rectangle (180 x 150)',
										'300x250' => 'IAB Medium Rectangle (300 x 250)',
										'120x90'  => 'IAB Button 1 (120 x 90)',
										'120x60'  => 'IAB Button 2 (120 x 60)',
										'125x125' => 'IAB Square Button (125 x 125)',
									)
								),
								'default_value'     => array(
									0 => '468x60',
								),
								'allow_null'        => 0,
								'multiple'          => 0,
								'ui'                => 1,
								'ajax'              => 0,
								'return_format'     => 'value',
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_auto_refresh_ads',
								'label'             => 'Auto Refresh Ads',
								'name'              => 'auto_refresh_ads',
								'type'              => 'true_false',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'message'           => '',
								'default_value'     => 0,
								'ui'                => 0,
								'ui_on_text'        => '',
								'ui_off_text'       => '',
							),
							array(
								'key'               => 'field_transition_effect',
								'label'             => 'Transition Effect',
								'name'              => 'transition_effect',
								'type'              => 'select',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => array(
									array(
										array(
											'field'    => 'field_auto_refresh_ads',
											'operator' => '==',
											'value'    => '1',
										),
									),
								),
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'choices'           => apply_filters(
									'wpadcenter_adzones_transition_effect_field_choices',
									array(
										'none'       => 'None',
										'fade'       => 'Fade',
										'fadeout'    => 'Fade Out',
										'scrollHorz' => 'Scroll Horizontal',
										'scrollVert' => 'Scroll Vertical',
									)
								),
								'default_value'     => array(
									0 => 'none',
								),
								'allow_null'        => 0,
								'multiple'          => 0,
								'ui'                => 1,
								'ajax'              => 0,
								'return_format'     => 'value',
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_transition_speed',
								'label'             => 'Transition Speed',
								'name'              => 'transition_speed',
								'type'              => 'number',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => array(
									array(
										array(
											'field'    => 'field_auto_refresh_ads',
											'operator' => '==',
											'value'    => '1',
										),
									),
								),
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => 500,
								'placeholder'       => '',
								'prepend'           => '',
								'append'            => '',
								'min'               => 500,
								'max'               => 60000,
								'step'              => 100,
							),
							array(
								'key'               => 'field_transition_delay',
								'label'             => 'Transition Delay',
								'name'              => 'transition_delay',
								'type'              => 'number',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => array(
									array(
										array(
											'field'    => 'field_auto_refresh_ads',
											'operator' => '==',
											'value'    => '1',
										),
									),
								),
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => 1000,
								'placeholder'       => '',
								'prepend'           => '',
								'append'            => '',
								'min'               => 1000,
								'max'               => 60000,
								'step'              => 100,
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-adzones',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'side',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);

			// Ad Zones – Extra Settings.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_adzones_extra',
					'title'                 => 'Extra Settings',
					'fields'                => apply_filters(
						'wpadcenter_adzones_extra_fields',
						array(
							array(
								'key'               => 'field_signup_link',
								'label'             => 'Signup Page Link',
								'name'              => 'signup_link',
								'type'              => 'url',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_default_banner',
								'label'             => 'Default Banner',
								'name'              => 'default_banner',
								'type'              => 'image',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'return_format'     => 'id',
								'preview_size'      => 'thumbnail',
								'library'           => 'all',
								'min_width'         => '',
								'min_height'        => '',
								'min_size'          => '',
								'max_width'         => '',
								'max_height'        => '',
								'max_size'          => '',
								'mime_types'        => '.gif, .jpg, .png, .swf, .bmp',
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-adzones',
							),
						),
					),
					'menu_order'            => 5,
					'position'              => 'side',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);

		}

		do_action( 'wpadcenter_after_adzones_acf_fields' );

		do_action( 'wpadcenter_before_advertisers_acf_fields' );

		if ( function_exists( 'acf_add_local_field_group' ) ) {
			// Advertisers – General.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_adverts_general',
					'title'                 => 'General',
					'fields'                => apply_filters(
						'wpadcenter_adverts_general_fields',
						array(
							array(
								'key'               => 'field_email',
								'label'             => 'Email',
								'name'              => 'email',
								'type'              => 'email',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
								'prepend'           => '',
								'append'            => '',
							),
							array(
								'key'               => 'field_adverts_status',
								'label'             => 'Status',
								'name'              => 'adverts_status',
								'type'              => 'select',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'choices'           => apply_filters(
									'wpadcenter_adverts_status_field_choices',
									array(
										'approved' => 'Approved',
										'pending'  => 'Pending',
										'declined' => 'Declined',
									)
								),
								'allow_null'        => 0,
								'mulitple'          => 0,
								'ui'                => 0,
								'default_value'     => array(
									0 => 'pending',
								),
								'return_format'     => 'value',
								'ajax'              => 0,
								'placeholder'       => 0,
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-adverts',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'acf_after_title',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);
		}

		do_action( 'wpadcenter_after_advertisers_acf_fields' );

		do_action( 'wpadcenter_before_campaigns_acf_fields' );

		if ( function_exists( 'acf_add_local_field_group' ) ) {
			// Campaigns - General.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_campaigns_general',
					'title'                 => 'General',
					'fields'                => apply_filters(
						'wpadcenter_campaigns_general_fields',
						array(
							array(
								'key'               => 'field_campaigns_description',
								'label'             => 'Description',
								'name'              => 'campaigns_description',
								'type'              => 'textarea',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
								'maxlength'         => '',
								'rows'              => '',
								'new_lines'         => '',
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-campaigns',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'acf_after_title',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);
			// Campaigns - Settings.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_campaigns_settings',
					'title'                 => 'Settings',
					'fields'                => apply_filters(
						'wpadcenter_campaigns_settings_fields',
						array(
							array(
								'key'               => 'field_campaigns_advertiser',
								'label'             => 'Advertiser',
								'name'              => 'campaigns_advertiser',
								'type'              => 'post_object',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'post_type'         => array(
									0 => 'wpadcenter-adverts',
								),
								'taxonomy'          => '',
								'allow_null'        => 0,
								'multiple'          => 0,
								'return_format'     => 'id',
								'ui'                => 1,
							),
							array(
								'key'               => 'field_start_date',
								'label'             => 'Start Date',
								'name'              => 'start_date',
								'type'              => 'date_picker',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'display_format'    => 'd/m/Y',
								'return_format'     => 'd/m/Y',
								'first_day'         => 1,
							),
							array(
								'key'               => 'field_end_date',
								'label'             => 'End Date',
								'name'              => 'end_date',
								'type'              => 'date_picker',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'display_format'    => 'd/m/Y',
								'return_format'     => 'd/m/Y',
								'first_day'         => 1,
							),
							array(
								'key'               => 'field_campaigns_status',
								'label'             => 'Status',
								'name'              => 'campaigns_status',
								'type'              => 'select',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'choices'           => apply_filters(
									'wpadcenter_campaigns_status_field_choices',
									array(
										'paused'    => 'Paused',
										'running'   => 'Running',
										'scheduled' => 'Scheduled',
										'finished'  => 'Finished',
									)
								),
								'allow_null'        => 0,
								'mulitple'          => 0,
								'ui'                => 0,
								'default_value'     => array(
									0 => 'running',
								),
								'return_format'     => 'value',
								'ajax'              => 0,
								'placeholder'       => 0,
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-campaigns',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'side',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);
		}

		do_action( 'wpadcenter_after_campaigns_acf_fields' );

		do_action( 'wpadcenter_before_banners_acf_fields' );

		if ( function_exists( 'acf_add_local_field_group' ) ) {
			// Banners - General.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_banners_general',
					'title'                 => 'General',
					'fields'                => apply_filters(
						'wpadcenter_banners_general_fields',
						array(
							array(
								'key'               => 'field_link_url',
								'label'             => 'Link URL',
								'name'              => 'link_url',
								'type'              => 'url',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_link_target',
								'label'             => 'Link Target',
								'name'              => 'link_target',
								'type'              => 'select',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'choices'           => apply_filters(
									'wpadcenter_banners_link_target_field_choices',
									array(
										'_blank'  => '_blank',
										'_self'   => '_self',
										'_parent' => '_parent',
										'_top'    => '_top',
									)
								),
								'default_value'     => array(
									0 => '_blank',
								),
								'allow_null'        => 0,
								'multiple'          => 0,
								'ui'                => 0,
								'return_format'     => 'value',
								'ajax'              => 0,
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_banner_type',
								'label'             => 'Banner Type',
								'name'              => 'banner_type',
								'type'              => 'select',
								'instructions'      => '',
								'required'          => 0,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'choices'           => apply_filters(
									'wpadcenter_banners_type_field_choices',
									array(
										'image'   => 'Image',
										'external_image_link' => 'External Image Link',
										'ad_code' => 'Ad Code',
									)
								),
								'default_value'     => array(
									0 => 'image',
								),
								'allow_null'        => 0,
								'multiple'          => 0,
								'ui'                => 0,
								'return_format'     => 'value',
								'ajax'              => 0,
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_image',
								'label'             => 'Image',
								'name'              => 'image',
								'type'              => 'image',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => array(
									array(
										array(
											'field'    => 'field_banner_type',
											'operator' => '==',
											'value'    => 'image',
										),
									),
								),
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'return_format'     => 'id',
								'preview_size'      => 'thumbnail',
								'library'           => 'all',
								'min_width'         => '',
								'min_height'        => '',
								'min_size'          => '',
								'max_width'         => '',
								'max_height'        => '',
								'max_size'          => '',
								'mime_types'        => '',
							),
							array(
								'key'               => 'field_image_link',
								'label'             => 'External Image Link',
								'name'              => 'image_link',
								'type'              => 'url',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => array(
									array(
										array(
											'field'    => 'field_banner_type',
											'operator' => '==',
											'value'    => 'external_image_link',
										),
									),
								),
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
							),
							array(
								'key'               => 'field_ad_code',
								'label'             => 'Ad Code',
								'name'              => 'ad_code',
								'type'              => 'textarea',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => array(
									array(
										array(
											'field'    => 'field_banner_type',
											'operator' => '==',
											'value'    => 'ad_code',
										),
									),
								),
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'default_value'     => '',
								'placeholder'       => '',
								'maxlength'         => '',
								'rows'              => '',
								'new_lines'         => '',
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-banners',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'acf_after_title',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);

			// Banners - Settings.
			acf_add_local_field_group(
				array(
					'key'                   => 'group_banners_settings',
					'title'                 => 'Banners - Settings',
					'fields'                => apply_filters(
						'wpadcenter_banners_settings_fields',
						array(
							array(
								'key'               => 'field_banners_advertiser',
								'label'             => 'Advertiser',
								'name'              => 'banners_advertiser',
								'type'              => 'post_object',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'post_type'         => array(
									0 => 'wpadcenter-adverts',
								),
								'taxonomy'          => '',
								'allow_null'        => 0,
								'multiple'          => 0,
								'return_format'     => 'id',
								'ui'                => 1,
							),
							array(
								'key'               => 'field_banners_campaign',
								'label'             => 'Campaign',
								'name'              => 'banners_campaign',
								'type'              => 'post_object',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'post_type'         => array(
									0 => 'wpadcenter-campaigns',
								),
								'taxonomy'          => '',
								'allow_null'        => 0,
								'multiple'          => 0,
								'return_format'     => 'id',
								'ui'                => 1,
							),
							array(
								'key'               => 'field_banners_ad_zone',
								'label'             => 'Ad Zone',
								'name'              => 'banners_ad_zone',
								'type'              => 'post_object',
								'instructions'      => '',
								'required'          => 1,
								'conditional_logic' => 0,
								'wrapper'           => array(
									'width' => '',
									'class' => '',
									'id'    => '',
								),
								'post_type'         => array(
									0 => 'wpadcenter-adzones',
								),
								'taxonomy'          => '',
								'allow_null'        => 0,
								'multiple'          => 0,
								'return_format'     => 'id',
								'ui'                => 1,
							),
						)
					),
					'location'              => array(
						array(
							array(
								'param'    => 'post_type',
								'operator' => '==',
								'value'    => 'wpadcenter-banners',
							),
						),
					),
					'menu_order'            => 1,
					'position'              => 'side',
					'style'                 => 'default',
					'label_placement'       => 'left',
					'instruction_placement' => 'field',
					'hide_on_screen'        => '',
					'active'                => true,
					'description'           => '',
				)
			);
		}

		do_action( 'wpadcenter_after_banners_acf_fields' );

	}

	/**
	 * Prints a combobox based on options and selected=match value.
	 *
	 * @since 1.0.0
	 * @param array  $options Array of options.
	 * @param string $selected Which of those options should be selected (allows just one; is case sensitive).
	 */
	public function print_combobox_options( $options, $selected ) {
		foreach ( $options as $key => $value ) {
			echo '<option value="' . esc_html( $value ) . '"';
			if ( $value === $selected ) {
				echo ' selected="selected"';
			}
			echo '>' . esc_html( $key ) . '</option>';
		}
	}

	/**
	 * Return transition effect options.
	 *
	 * @since 1.0.0
	 */
	public function get_transition_effect_options() {
		return apply_filters(
			'wpadcenter_transition_effect_options',
			array(
				__( 'None', 'wpadcenter' )              => 'none',
				__( 'Fade', 'wpadcenter' )              => 'fade',
				__( 'Fade Out', 'wpadcenter' )          => 'fadeout',
				__( 'Scroll Horizontal', 'wpadcenter' ) => 'scrollHorz',
				__( 'Scroll Vertical', 'wpadcenter' )   => 'scrollVert',
			)
		);
	}

	/**
	 * Define CPT's messages.
	 *
	 * @since 1.0.0
	 * @param Array $messages Default Post Messages.
	 * @return mixed
	 */
	public function wpadcenter_cpt_update_messages( $messages ) {

		$wpadcenter_cpts = apply_filters(
			'wpadcenter_cpts',
			array(
				'wpadcenter-adzones'   => 'Ad Zone',
				'wpadcenter-adverts'   => 'Advertiser',
				'wpadcenter-campaigns' => 'Campaign',
				'wpadcenter-banners'   => 'Banner',
			)
		);

		foreach ( $wpadcenter_cpts as $cpt => $val ) {
			$messages[ $cpt ] = array(
				0 => '', // Unused. Messages start at index 1.
				/* translators: %s: custom post type */
				1 => sprintf( __( '%s updated.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				4 => sprintf( __( '%s updated.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				6 => sprintf( __( '%s saved.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				7 => sprintf( __( '%s saved.', 'wpadcenter' ), $val ),
				/* translators: %s: custom post type */
				8 => sprintf( __( '%s saved.', 'wpadcenter' ), $val ),
			);
		}

		return $messages;
	}

	/**
	 * Remove quick edit option for CPT's.
	 *
	 * @since 1.0.0
	 * @param Array  $actions Post actions.
	 * @param Object $post Post.
	 * @return mixed
	 */
	public function wpadcenter_remove_cpt_quick_edit_action( $actions, $post ) {
		$wpadcenter_cpts = array( 'wpadcenter-adzones', 'wpadcenter-adverts', 'wpadcenter-campaigns', 'wpadcenter-banners' );
		if ( in_array( $post->post_type, $wpadcenter_cpts, true ) ) {
			unset( $actions['inline hide-if-no-js'] );
		}
		$post_type_object = get_post_type_object( $post->post_type );
		$title            = _draft_or_post_title();
		if ( 'wpadcenter-adverts' === $post->post_type ) {
			if ( 'adc-pending' === $post->post_status ) {
				$actions['adc-approve'] = sprintf(
					/* translators: 1: href link 2: aria label 3: Text */
					'<a href="%s" aria-label="%s">%s</a>',
					wp_nonce_url( admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=adc-approve', $post->ID ) ), 'adc-approve-post_' . $post->ID ),
					esc_attr(
						sprintf(
							/* translators: %s: post title */
							__( 'Approve &#8220;%s&#8221;', 'wpadcenter' ),
							$title
						)
					),
					__( 'Approve', 'wpadcenter' )
				);
				$actions['adc-decline'] = sprintf(
					/* translators: 1: href link 2: aria label 3: Text */
					'<a href="%s" aria-label="%s">%s</a>',
					wp_nonce_url( admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=adc-decline', $post->ID ) ), 'adc-decline-post_' . $post->ID ),
					esc_attr(
						sprintf(
							/* translators: %s: post title */
							__( 'Decline &#8220;%s&#8221;', 'wpadcenter' ),
							$title
						)
					),
					__( 'Decline', 'wpadcenter' )
				);
			}
		}
		if ( 'wpadcenter-campaigns' === $post->post_type ) {
			if ( 'adc-paused' === $post->post_status ) {
				$actions['adc-run'] = sprintf(
					/* translators: 1: href link 2: aria label 3: Text */
					'<a href="%s" aria-label="%s">%s</a>',
					wp_nonce_url( admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=adc-run', $post->ID ) ), 'adc-run-post_' . $post->ID ),
					esc_attr(
						sprintf(
							/* translators: %s: post title */
							__( 'Re-Run &#8220;%s&#8221;', 'wpadcenter' ),
							$title
						)
					),
					__( 'Re-Run', 'wpadcenter' )
				);
			}
			if ( 'adc-running' === $post->post_status ) {
				$actions['adc-pause'] = sprintf(
					/* translators: 1: href link 2: aria label 3: Text */
					'<a href="%1s" aria-label="%2s">%3s</a>',
					wp_nonce_url( admin_url( sprintf( $post_type_object->_edit_link . '&amp;action=adc-pause', $post->ID ) ), 'adc-pause-post_' . $post->ID ),
					esc_attr(
						sprintf(
							/* translators: %s: post title */
							__( 'Pause &#8220;%s&#8221;', 'wpadcenter' ),
							$title
						)
					),
					__( 'Pause', 'wpadcenter' )
				);
			}
		}
		return $actions;
	}

	/**
	 * Remove bulk edit options for CPT's.
	 *
	 * @since 1.0.0
	 * @param Array $actions Post actions.
	 * @return mixed
	 */
	public function wpadcenter_remove_cpt_bulk_edit_action( $actions ) {
		unset( $actions['edit'] );
		return $actions;
	}

	/**
	 * Hides publishing actions for CPT's.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_hide_cpt_publishing_actions() {
		global $post;
		$wpadcenter_cpts = array( 'wpadcenter-adzones', 'wpadcenter-adverts', 'wpadcenter-campaigns', 'wpadcenter-banners' );
		if ( in_array( $post->post_type, $wpadcenter_cpts, true ) ) {
			echo '
                <style type="text/css">
                    #misc-publishing-actions,
                    #minor-publishing-actions,
                    #delete-action{
                        display:none;
                    }
                </style>
            ';
		}
	}

	/**
	 * Updated CPT's title text.
	 *
	 * @since 1.0.0
	 * @param String $title Post title.
	 * @return string
	 */
	public function wpadcenter_change_cpt_default_title( $title ) {
		global $current_screen;

		if ( 'wpadcenter-adzones' === $current_screen->post_type ) {
			$title = 'Ad Zone Name';
		} elseif ( 'wpadcenter-adverts' === $current_screen->post_type ) {
			$title = 'Advertiser Name';
		} elseif ( 'wpadcenter-campaigns' === $current_screen->post_type ) {
			$title = 'Campaign Name';
		} elseif ( 'wpadcenter-banners' === $current_screen->post_type ) {
			$title = 'Banner Name';
		}

		return $title;
	}

	/**
	 * Updated CPT's publish button text.
	 *
	 * @since 1.0.0
	 * @param String $translation Post translated text.
	 * @param String $text Post publish button text.
	 * @return string
	 */
	public function wpadcenter_change_cpt_publish_button( $translation, $text ) {
		if ( 'wpadcenter-adzones' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return 'Save Ad Zone';
			} elseif ( 'Update' === $text ) {
				return 'Update Ad Zone';
			}
		} elseif ( 'wpadcenter-adverts' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return 'Save Advertiser';
			} elseif ( 'Update' === $text ) {
				return 'Update Advertiser';
			}
		} elseif ( 'wpadcenter-campaigns' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return 'Save Campaign';
			} elseif ( 'Update' === $text ) {
				return 'Update Campaign';
			}
		} elseif ( 'wpadcenter-banners' === get_post_type() ) {
			if ( 'Publish' === $text ) {
				return 'Save Banner';
			} elseif ( 'Update' === $text ) {
				return 'Update Banner';
			}
		}
		return $translation;
	}

	/**
	 * Manage Ad Zones columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_adzones_columns() {
		global $current_screen;
		if ( 'wpadcenter-adzones' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'               => '<input type="checkbox" />',
			'title'            => 'Name',
			'size'             => 'Size',
			'auto_refresh_ads' => 'Auto Refresh Ads',
			'shortcode'        => 'Shortcode',
			'date'             => 'Date',
		);
		return $columns;
	}

	/**
	 * Manage Advertisers columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_adverts_columns() {
		global $current_screen;
		if ( 'wpadcenter-adverts' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'             => '<input type="checkbox" />',
			'title'          => 'Name',
			'email'          => 'Email',
			'adverts_status' => 'Status',
			'date'           => 'Date',
		);
		return $columns;
	}

	/**
	 * Manage Campaigns columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_campaigns_columns() {
		global $current_screen;
		if ( 'wpadcenter-campaigns' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'               => '<input type="checkbox" />',
			'title'            => 'Name',
			'advertiser'       => 'Advertiser',
			'start_date'       => 'Start Date',
			'end_date'         => 'End Date',
			'campaigns_status' => 'Status',
		);
		return $columns;
	}

	/**
	 * Manage Banners columns
	 *
	 * @since 1.0.0
	 * @return array|void
	 */
	public function wpadcenter_manage_edit_banners_columns() {
		global $current_screen;
		if ( 'wpadcenter-banners' !== $current_screen->post_type ) {
			return;
		}
		$columns = array(
			'cb'          => '<input type="checkbox" />',
			'title'       => 'Name',
			'banner_type' => 'Type',
			'advertiser'  => 'Advertiser',
			'campaign'    => 'Campaign',
			'ad_zone'     => 'Ad Zone',
			'date'        => 'Date',
		);
		return $columns;
	}

	/**
	 * Render Ad Zones columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_adzones_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-adzones' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'size':
				echo esc_attr( get_field( 'template', $post->ID ) );
				break;
			case 'auto_refresh_ads':
				$auto_refresh_ads = get_field( 'auto_refresh_ads', $post->ID );
				if ( isset( $auto_refresh_ads ) && true === $auto_refresh_ads ) {
					esc_attr_e( 'Yes', 'wpadcenter' );
				} else {
					esc_attr_e( 'No', 'wpadcenter' );
				}
				break;
			case 'shortcode':
				echo '[wpadcenter id=' . esc_attr( $post->ID ) . ']';
				break;
		}
	}

	/**
	 * Render Advertisers columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_adverts_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-adverts' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'email':
				echo esc_attr( get_field( 'email', $post->ID ) );
				break;
			case 'adverts_status':
				echo esc_attr( ucfirst( get_field( 'adverts_status', $post->ID ) ) );
				break;
		}
	}

	/**
	 * Render Campaigns columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_campaigns_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-campaigns' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'advertiser':
				$advertiser = get_field( 'campaigns_advertiser', $post->ID );
				echo '<a class="row-title" href="' . esc_url( get_edit_post_link( $advertiser ) ) . '" aria-label="' . esc_attr( get_the_title( $advertiser ) ) . ' (Edit)">' . esc_attr( get_the_title( $advertiser ) ) . '</a>';
				break;
			case 'start_date':
				echo esc_attr( get_field( 'start_date', $post->ID ) );
				break;
			case 'end_date':
				echo esc_attr( get_field( 'end_date', $post->ID ) );
				break;
			case 'campaigns_status':
				echo esc_attr( ucfirst( get_field( 'campaigns_status', $post->ID ) ) );
				break;
		}
	}

	/**
	 * Render Banners columns.
	 *
	 * @since 1.0.0
	 * @param String $column Column.
	 */
	public function wpadcenter_manage_custom_banners_columns( $column ) {
		global $post;
		global $current_screen;

		if ( 'wpadcenter-banners' !== $current_screen->post_type ) {
			return;
		}

		switch ( $column ) {
			case 'banner_type':
				echo esc_attr( ucwords( str_replace( '_', ' ', get_field( 'banner_type', $post->ID ) ) ) );
				break;
			case 'advertiser':
				$advertiser = get_field( 'banners_advertiser', $post->ID );
				echo '<a class="row-title" href="' . esc_url( get_edit_post_link( $advertiser ) ) . '" aria-label="' . esc_attr( get_the_title( $advertiser ) ) . ' (Edit)">' . esc_attr( get_the_title( $advertiser ) ) . '</a>';
				break;
			case 'campaign':
				$campaign = get_field( 'banners_campaign', $post->ID );
				echo '<a class="row-title" href="' . esc_url( get_edit_post_link( $campaign ) ) . '" aria-label="' . esc_attr( get_the_title( $campaign ) ) . ' (Edit)">' . esc_attr( get_the_title( $campaign ) ) . '</a>';
				break;
			case 'ad_zone':
				$ad_zone = get_field( 'banners_ad_zone', $post->ID );
				echo '<a class="row-title" href="' . esc_url( get_edit_post_link( $ad_zone ) ) . '" aria-label="' . esc_attr( get_the_title( $ad_zone ) ) . ' (Edit)">' . esc_attr( get_the_title( $ad_zone ) ) . '</a>';
				break;
		}
	}

	/**
	 * Save Campaigns CPT.
	 *
	 * @since 1.0.0
	 * @param int $post_id Post ID.
	 */
	public function wpadcenter_campaigns_save_custom_post( $post_id ) {
		$post = get_post( $post_id );
		if ( 'wpadcenter-campaigns' !== $post->post_type ) {
			return;
		}
		if ( $post_id ) {
			$campaigns_status = get_field( 'campaigns_status', $post_id );
			if ( $campaigns_status ) {
				$today      = date( 'd/m/Y' );
				$start_date = get_field( 'start_date', $post_id );
				$end_date   = get_field( 'end_date', $post_id );
				if ( 'paused' !== $campaigns_status ) {
					$campaigns_status = 'running';
					if ( ! empty( $start_date ) && $start_date > $today ) {
						$campaigns_status = 'scheduled';
					}
					if ( ! empty( $end_date ) && $end_date < $today ) {
						$campaigns_status = 'finished';
					}
					update_field( 'campaigns_status', $campaigns_status, $post_id );
				}
				if ( 'trash' !== $post->post_status ) {
					remove_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
					wp_update_post(
						array(
							'ID'          => $post_id,
							'post_status' => 'adc-' . trim( $campaigns_status ),
						)
					);
					add_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
				}
			}
		}
	}

	/**
	 * Save Advertisers CPT.
	 *
	 * @since 1.0.0
	 * @param int $post_id Post ID.
	 */
	public function wpadcenter_advertisers_save_custom_post( $post_id ) {
		$post = get_post( $post_id );
		if ( 'wpadcenter-adverts' !== $post->post_type ) {
			return;
		}
		if ( $post_id ) {
			$adverts_status = get_field( 'adverts_status', $post_id );
			if ( $adverts_status && 'trash' !== $post->post_status ) {
				remove_action( 'save_post', array( $this, 'wpadcenter_advertisers_save_custom_post' ) );
				wp_update_post(
					array(
						'ID'          => $post_id,
						'post_status' => 'adc-' . trim( $adverts_status ),
					)
				);
				add_action( 'save_post', array( $this, 'wpadcenter_advertisers_save_custom_post' ) );
			}
			$adverts_email = get_field( 'email', $post_id );
			if ( $adverts_email ) {
				if ( ! get_user_by( 'email', $adverts_email ) ) {
					$user                 = array();
					$user['user_login']   = $post->post_name;
					$user['user_pass']    = wp_generate_password( 12 );
					$user['user_email']   = $adverts_email;
					$user['display_name'] = get_the_title( $post );
					$user['role']         = 'subscriber';
					if ( is_wp_error( wp_insert_user( $user ) ) ) {
						return;
					}
				}
			}
		}
	}

	/**
	 * Post row action - Approve advertiser.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_approve_advertisers() {
		$sendback = wp_get_referer();
		$sendback = remove_query_arg( array( 'advertiser_approved' ), $sendback );

		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
		} else {
			$post_id = 0;
		}
		if ( $post_id ) {
			$post = get_post( $post_id );
		}
		if ( $post ) {
			$post_type        = $post->post_type;
			$post_type_object = get_post_type_object( $post_type );
		}
		check_admin_referer( 'adc-approve-post_' . $post_id );
		if ( ! $post ) {
			wp_die( esc_attr__( 'The advertiser you are trying to approve no longer exists.', 'wpadcenter' ) );
		}

		if ( ! $post_type_object ) {
			wp_die( esc_attr__( 'Invalid post type.', 'wpadcenter' ) );
		}

		update_field( 'adverts_status', 'approved', $post_id );
		$post->post_status = 'adc-approved';

		remove_action( 'save_post', array( $this, 'wpadcenter_advertisers_save_custom_post' ) );
		$result = wp_update_post( $post );
		add_action( 'save_post', array( $this, 'wpadcenter_advertisers_save_custom_post' ) );

		if ( $result ) {
			$sendback = add_query_arg( 'advertiser_approved', true, $sendback );
		} else {
			$sendback = add_query_arg( 'advertiser_approved', false, $sendback );
		}
		wp_safe_redirect( $sendback ? $sendback : admin_url( 'edit.php?post_type=wpadcenter-adverts' ) );
		exit;
	}

	/**
	 * Post row action - Decline advertiser.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_decline_advertisers() {
		$sendback = wp_get_referer();
		$sendback = remove_query_arg( array( 'advertiser_declined' ), $sendback );

		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
		} else {
			$post_id = 0;
		}
		if ( $post_id ) {
			$post = get_post( $post_id );
		}
		if ( $post ) {
			$post_type        = $post->post_type;
			$post_type_object = get_post_type_object( $post_type );
		}
		check_admin_referer( 'adc-decline-post_' . $post_id );
		if ( ! $post ) {
			wp_die( esc_attr__( 'The advertiser you are trying to decline no longer exists.', 'wpadcenter' ) );
		}

		if ( ! $post_type_object ) {
			wp_die( esc_attr__( 'Invalid post type.', 'wpadcenter' ) );
		}

		update_field( 'adverts_status', 'declined', $post_id );
		$post->post_status = 'adc-declined';

		remove_action( 'save_post', array( $this, 'wpadcenter_advertisers_save_custom_post' ) );
		$result = wp_update_post( $post );
		add_action( 'save_post', array( $this, 'wpadcenter_advertisers_save_custom_post' ) );

		if ( $result ) {
			$sendback = add_query_arg( 'advertiser_declined', true, $sendback );
		} else {
			$sendback = add_query_arg( 'advertiser_declined', false, $sendback );
		}
		wp_safe_redirect( $sendback ? $sendback : admin_url( 'edit.php?post_type=wpadcenter-adverts' ) );
		exit;
	}

	/**
	 * Post row action - Re-run campaign.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_re_run_campaigns() {
		$sendback = wp_get_referer();
		$sendback = remove_query_arg( array( 'campaign_running' ), $sendback );

		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
		} else {
			$post_id = 0;
		}
		if ( $post_id ) {
			$post = get_post( $post_id );
		}
		if ( $post ) {
			$post_type        = $post->post_type;
			$post_type_object = get_post_type_object( $post_type );
		}
		check_admin_referer( 'adc-run-post_' . $post_id );
		if ( ! $post ) {
			wp_die( esc_attr__( 'The campaign you are trying to re-run no longer exists.', 'wpadcenter' ) );
		}

		if ( ! $post_type_object ) {
			wp_die( esc_attr__( 'Invalid post type.', 'wpadcenter' ) );
		}

		update_field( 'campaigns_status', 'running', $post_id );
		$post->post_status = 'adc-running';

		$result = wp_update_post( $post );

		if ( $result ) {
			$sendback = add_query_arg( 'campaign_running', true, $sendback );
		} else {
			$sendback = add_query_arg( 'campaign_running', false, $sendback );
		}
		wp_safe_redirect( $sendback ? $sendback : admin_url( 'edit.php?post_type=wpadcenter-campaigns' ) );
		exit;
	}

	/**
	 * Post row action - Pause campaign.
	 *
	 * @since 1.0.0
	 */
	public function wpadcenter_pause_campaigns() {
		$sendback = wp_get_referer();
		$sendback = remove_query_arg( array( 'campaign_paused' ), $sendback );

		if ( isset( $_GET['post'] ) ) {
			$post_id = (int) $_GET['post'];
		} else {
			$post_id = 0;
		}
		if ( $post_id ) {
			$post = get_post( $post_id );
		}
		if ( $post ) {
			$post_type        = $post->post_type;
			$post_type_object = get_post_type_object( $post_type );
		}
		check_admin_referer( 'adc-pause-post_' . $post_id );
		if ( ! $post ) {
			wp_die( esc_attr__( 'The campaign you are trying to pause no longer exists.', 'wpadcenter' ) );
		}

		if ( ! $post_type_object ) {
			wp_die( esc_attr__( 'Invalid post type.', 'wpadcenter' ) );
		}

		update_field( 'campaigns_status', 'paused', $post_id );
		$post->post_status = 'adc-paused';

		remove_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );
		$result = wp_update_post( $post );
		add_action( 'save_post', array( $this, 'wpadcenter_campaigns_save_custom_post' ) );

		if ( $result ) {
			$sendback = add_query_arg( 'campaign_paused', true, $sendback );
		} else {
			$sendback = add_query_arg( 'campaign_paused', false, $sendback );
		}
		wp_safe_redirect( $sendback ? $sendback : admin_url( 'edit.php?post_type=wpadcenter-campaigns' ) );
		exit;
	}

	/**
	 * Return custom status for Adverts CPT.
	 *
	 * @since 1.0.0
	 * @param string $key Status.
	 * @return mixed|void
	 */
	public function get_adverts_status( $key = '' ) {
		$adverts_status = apply_filters(
			'wpadcenter_adverts_status',
			array(
				'adc-approved' => __( 'Approved', 'wpadcenter' ),
				'adc-pending'  => __( 'Pending', 'wpadcenter' ),
				'adc-declined' => __( 'Declined', 'wpadcenter' ),
			)
		);
		return '' !== $key ? $adverts_status[ $key ] : $adverts_status;
	}

	/**
	 * Return custom status for Campaigns CPT.
	 *
	 * @since 1.0.0
	 * @param string $key Status.
	 * @return mixed|void
	 */
	public function get_campaigns_status( $key = '' ) {
		$campaigns_status = apply_filters(
			'wpadcenter_campaigns_status',
			array(
				'adc-paused'    => __( 'Paused', 'wpadcenter' ),
				'adc-running'   => __( 'Running', 'wpadcenter' ),
				'adc-scheduled' => __( 'Scheduled', 'wpadcenter' ),
				'adc-finished'  => __( 'Finished', 'wpadcenter' ),
			)
		);
		return '' !== $key ? $campaigns_status[ $key ] : $campaigns_status;
	}

	/**
	 * Convert CPT's custom post status.
	 *
	 * @since 1.0.0
	 * @param String $post_status Post Status.
	 * @param Object $post Post object.
	 * @return mixed|void
	 */
	public function wpadcenter_acf_get_post_status( $post_status, $post ) {
		if ( 'auto-draft' !== $post_status && 'trash' !== $post_status ) {
			if ( isset( $post->post_type ) ) {
				switch ( $post->post_type ) {
					case 'wpadcenter-adverts':
						$post_status = $this->get_adverts_status( $post_status );
						break;
					case 'wpadcenter-campaigns':
						$post_status = $this->get_campaigns_status( $post_status );
						break;
				}
			}
		}
		return $post_status;
	}

	/**
	 * Filter Post object field.
	 *
	 * @since 1.0.0
	 * @param Array $args Filter arguments.
	 * @param Array $field Field.
	 * @param int   $post_id Post ID.
	 * @return mixed
	 */
	public function wpadcenter_acf_filter_post_query( $args, $field, $post_id ) {
		if ( isset( $args['post_type'][0] ) ) {
			$post_type = $args['post_type'][0];
			if ( 'wpadcenter-adverts' === $post_type ) {
				$args['post_status'] = array( 'adc-approved' );
			}
		}
		return $args;
	}

	/**
	 * Filter Campaign field for Banners CPT.
	 *
	 * @since 1.0.0
	 * @param Array $args Filter arguments.
	 * @param Array $field Field.
	 * @param int   $post_id Post ID.
	 * @return mixed
	 */
	public function wpadcenter_acf_banners_campaign_filter_post_query( $args, $field, $post_id ) {
		if ( isset( $_POST['advertiser'] ) ) { // phpcs:ignore CSRF ok
			$advertiser         = sanitize_text_field( wp_unslash( $_POST['advertiser'] ) ); // phpcs:ignore CSRF ok
			$args['meta_key']   = 'campaigns_advertiser'; // phpcs:ignore slow query
			$args['meta_value'] = $advertiser; // phpcs:ignore slow query
		}
		return $args;
	}

	/**
	 * Returns ACF Settings URL.
	 *
	 * @since 1.0.0
	 * @param  string $url Settings URL.
	 * @return string $url.
	 */
	public function wpadcenter_acf_settings_url( $url ) {
		if ( ! get_option( 'wpadcenter_pro_active' ) ) {
			$url = WPADCENTER_PLUGIN_ACF_URL;
		}
		return $url;
	}

	/**
	 * Show Admin Menu for ACF.
	 *
	 * @since 1.0.0
	 * @param  bool $show_admin Show Admin flag.
	 * @return bool $show_admin
	 */
	public function wpadcenter_acf_settings_show_admin( $show_admin ) {
		if ( ! get_option( 'wpadcenter_pro_active' ) ) {
			$show_admin = false;
		}
		return $show_admin;
	}
}
