<?php
/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://club.wpeka.com/
 * @since      1.0.0
 *
 * @package    Wpadcenter
 * @subpackage Wpadcenter/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wpadcenter
 * @subpackage Wpadcenter/includes
 * @author     WPEka Club <support@wpeka.com>
 */
class Wpadcenter {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wpadcenter_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The currently stored option settings of the plugin.
	 *
	 * @since 1.0.0
	 * @access private
	 * @var array $stored_options The stored option settings of the plugin.
	 */
	private static $stored_options = array();

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'WPADCENTER_VERSION' ) ) {
			$this->version = WPADCENTER_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'wpadcenter';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wpadcenter_Loader. Orchestrates the hooks of the plugin.
	 * - Wpadcenter_I18n. Defines internationalization functionality.
	 * - Wpadcenter_Admin. Defines all hooks for the admin area.
	 * - Wpadcenter_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wpadcenter-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wpadcenter-public.php';

		/**
		 * The class responsible for defining all actions that occur in the widget area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wpadcenter-widget.php';

		/**
		 * The class responsible for defining all actions that occur in the acf.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/libraries/acf/acf.php';

		$this->loader = new Wpadcenter_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wpadcenter_I18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wpadcenter_I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wpadcenter_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		// Define action hooks.
		$this->loader->add_action( 'admin_menu', $plugin_admin, 'wpadcenter_admin_menu' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_cpt' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_cpt_adverts_status' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_cpt_campaigns_status' );
		$this->loader->add_action( 'init', $plugin_admin, 'wpadcenter_register_block_type' );
		$this->loader->add_action( 'widgets_init', $plugin_admin, 'wpadcenter_register_widget' );

		$this->loader->add_action( 'acf/init', $plugin_admin, 'wpadcenter_add_acf_fields' );
		$this->loader->add_action( 'acf/input/admin_enqueue_scripts', $plugin_admin, 'wpadcenter_enqueue_acf_scripts' );

		$this->loader->add_action( 'manage_edit-wpadcenter-adzones_columns', $plugin_admin, 'wpadcenter_manage_edit_adzones_columns' );
		$this->loader->add_action( 'manage_edit-wpadcenter-adverts_columns', $plugin_admin, 'wpadcenter_manage_edit_adverts_columns' );
		$this->loader->add_action( 'manage_edit-wpadcenter-campaigns_columns', $plugin_admin, 'wpadcenter_manage_edit_campaigns_columns' );
		$this->loader->add_action( 'manage_edit-wpadcenter-banners_columns', $plugin_admin, 'wpadcenter_manage_edit_banners_columns' );

		$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_adzones_columns' );
		$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_adverts_columns' );
		$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_campaigns_columns' );
		$this->loader->add_action( 'manage_posts_custom_column', $plugin_admin, 'wpadcenter_manage_custom_banners_columns' );

		$this->loader->add_action( 'save_post', $plugin_admin, 'wpadcenter_campaigns_save_custom_post' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'wpadcenter_advertisers_save_custom_post' );

		$this->loader->add_action( 'post_action_adc-approve', $plugin_admin, 'wpadcenter_approve_advertisers' );
		$this->loader->add_action( 'post_action_adc-decline', $plugin_admin, 'wpadcenter_decline_advertisers' );
		$this->loader->add_action( 'post_action_adc-run', $plugin_admin, 'wpadcenter_re_run_campaigns' );
		$this->loader->add_action( 'post_action_adc-pause', $plugin_admin, 'wpadcenter_pause_campaigns' );

		$this->loader->add_action( 'admin_head-post.php', $plugin_admin, 'wpadcenter_hide_cpt_publishing_actions' );
		$this->loader->add_action( 'admin_head-post-new.php', $plugin_admin, 'wpadcenter_hide_cpt_publishing_actions' );
		$this->loader->add_action( 'post_updated_messages', $plugin_admin, 'wpadcenter_cpt_update_messages' );

		// Define content filters.
		$this->loader->add_filter( 'enter_title_here', $plugin_admin, 'wpadcenter_change_cpt_default_title' );
		$this->loader->add_filter( 'gettext', $plugin_admin, 'wpadcenter_change_cpt_publish_button', 10, 2 );
		$this->loader->add_filter( 'post_row_actions', $plugin_admin, 'wpadcenter_remove_cpt_quick_edit_action', 10, 2 );
		$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-adzones', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );
		$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-adverts', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );
		$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-campaigns', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );
		$this->loader->add_filter( 'bulk_actions-edit-wpadcenter-banners', $plugin_admin, 'wpadcenter_remove_cpt_bulk_edit_action', 10, 1 );

		$this->loader->add_filter( 'get_post_status', $plugin_admin, 'wpadcenter_acf_get_post_status', 10, 2 );
		$this->loader->add_filter( 'acf/settings/url', $plugin_admin, 'wpadcenter_acf_settings_url', 10, 1 );
		$this->loader->add_filter( 'acf/settings/show_admin', $plugin_admin, 'wpadcenter_acf_settings_show_admin', 10, 1 );
		$this->loader->add_filter( 'acf/fields/post_object/query', $plugin_admin, 'wpadcenter_acf_filter_post_query', 10, 3 );
		$this->loader->add_filter( 'acf/fields/post_object/query/key=field_banners_campaign', $plugin_admin, 'wpadcenter_acf_banners_campaign_filter_post_query', 10, 3 );
		$this->loader->add_filter( 'plugin_action_links_' . WPADCENTER_PLUGIN_BASENAME, $plugin_admin, 'wpadcenter_plugin_action_links' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wpadcenter_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

	}

	/**
	 * Returns default settings.
	 * If you override the settings here, be ultra careful to use escape characters.
	 *
	 * @since 1.0.0
	 * @param string $key Return default settings for particular key.
	 * @return mixed|void
	 */
	public static function wpadcenter_get_default_settings( $key = '' ) {
		$settings = array(
			// General settings.
			'notification'         => false,
			'notification_email'   => '',
			'notification_name'    => '',

			'auto_refresh'         => false,
			'transition_effect'    => 'none',
			'transition_speed'     => '500',
			'transition_delay'     => '1000',

			'geo_location'         => 'none',

			// Payments settings.
			'paypal'               => false,
			'paypal_email'         => '',
			'paypal_currency'      => 'USD',
			'paypal_sandbox'       => false,

			'stripe'               => false,
			'stripe_secret_key'    => '',
			'stripe_publisher_key' => '',
			'stripe_currency'      => 'USD',
		);
		$settings = apply_filters( 'wpadcenter_default_settings', $settings );
		return '' !== $key ? $settings[ $key ] : $settings;
	}

	/**
	 * Returns sanitised content based on field-specific rules defined here
	 * used for both read AND write operations.
	 *
	 * @since 1.0.0
	 * @param string $key Key for the setting.
	 * @param string $value Value for the setting.
	 *
	 * @return bool|null|string
	 */
	public static function wpadcenter_sanitise_settings( $key, $value ) {
		$ret = null;
		switch ( $key ) {
			// Convert all boolean values from text to bool.
			case 'notification':
			case 'auto_refresh':
			case 'paypal':
			case 'paypal_sandbox':
			case 'stripe':
				if ( 'true' === $value || true === $value ) {
					$ret = true;
				} elseif ( 'false' === $value || false === $value ) {
					$ret = false;
				} else {
					// Unexpected value returned from radio button, go fix the HTML.
					// Failover = assign null.
					$ret = 'fffffff';
				}
				break;
			default:
				$ret = sanitize_text_field( $value );
				break;
		}
		if ( 'fffffff' === $ret ) {
			$ret = false;
		}
		return $ret;
	}

	/**
	 * Get current settings.
	 *
	 * @since 1.0.0
	 *
	 * @return array|mixed
	 */
	public static function wpadcenter_get_settings() {
		$settings             = self::wpadcenter_get_default_settings();
		self::$stored_options = get_option( WPADCENTER_SETTINGS_FIELD );
		if ( ! empty( self::$stored_options ) ) {
			foreach ( self::$stored_options as $key => $option ) {
				$settings[ $key ] = self::wpadcenter_sanitise_settings( $key, $option );
			}
		}
		update_option( WPADCENTER_SETTINGS_FIELD, $settings );
		return $settings;
	}

	/**
	 * Generate tab head for settings page,
	 * method will translate the string to current language.
	 *
	 * @since 1.0.0
	 *
	 * @param array $title_arr Tab labels.
	 */
	public static function wpadcenter_generate_settings_tabhead( $title_arr ) {
		foreach ( $title_arr as $k => $v ) {
			if ( is_array( $v ) ) {
				$v = ( isset( $v[2] ) ? $v[2] : '' ) . esc_attr( $v[0] ) . ' ' . ( isset( $v[1] ) ? $v[1] : '' );
			} else {
				$v = esc_attr( $v );
			}
			?>
			<a class="nav-tab" href="#<?php echo esc_html( $k ); ?>"><?php echo esc_html( $v ); ?></a>
			<?php
		}
	}

	/**
	 * Envelope settings tab content with tab div.
	 *
	 * @since 1.0.0
	 * @param string $view_file View file.
	 * @param string $target_id Target tab id.
	 */
	public static function wpadcenter_envelope_settings_tab( $view_file = '', $target_id ) {
		$the_options = self::wpadcenter_get_settings();
		?>
		<div class="wpadcenter-tab-content" data-id="<?php echo esc_attr( $target_id ); ?>">
			<?php
			if ( '' !== $view_file && file_exists( $view_file ) ) {
				include_once $view_file;
			}
			include plugin_dir_path( WPADCENTER_PLUGIN_FILENAME ) . 'admin/views/admin-display-save-button.php';
			?>
		</div>
		<?php
	}

	/**
	 * Return IP address of the user.
	 *
	 * @since 1.0.0
	 * @return string
     * @phpcs:disable
     */
    public static function wpadcenter_get_user_ip() {
        if ( isset( $_SERVER['HTTP_CLIENT_IP'] ) ) {
            $ipaddress = $_SERVER['HTTP_CLIENT_IP'];
        } elseif ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
            $ipaddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } elseif ( isset( $_SERVER['HTTP_X_FORWARDED'] ) ) {
            $ipaddress = $_SERVER['HTTP_X_FORWARDED'];
        } elseif ( isset( $_SERVER['HTTP_FORWARDED_FOR'] ) ) {
            $ipaddress = $_SERVER['HTTP_FORWARDED_FOR'];
        } elseif ( isset( $_SERVER['HTTP_FORWARDED'] ) ) {
            $ipaddress = $_SERVER['HTTP_FORWARDED'];
        } elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
            $ipaddress = $_SERVER['REMOTE_ADDR'];
        } else {
            $ipaddress = 'UNKNOWN';
        }
        return $ipaddress;
    }

    /**
     * Return country code for the user IP.
     *
     * @since 1.0.0
     * @return bool
	 * @phpcs:enable
	 */
	public static function wpadcenter_get_geo_country_code() {
		include_once plugin_dir_path( __FILE__ ) . 'libraries/geoip/geoip.inc';
		$user_ip      = self::wpadcenter_get_user_ip();
		$country_code = '';
		if ( $user_ip && 'UNKNOWN' !== $user_ip ) {
			if ( filter_var( $user_ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6 ) ) {
				$gi = new WpadcenterGeoIP();
				$gi->geoip_open( plugin_dir_path( __FILE__ ) . 'libraries/geoip/GeoIPv6.dat', GEOIP_STANDARD );
				$country_code = $gi->geoip_country_code_by_addr_v6( $user_ip );
			} elseif ( filter_var( $user_ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4 ) ) {
				$gi = new WpadcenterGeoIP();
				$gi->geoip_open( plugin_dir_path( __FILE__ ) . 'libraries/geoip/GeoIP.dat', GEOIP_STANDARD );
				$country_code = $gi->geoip_country_code_by_addr( $user_ip );
			}
			if ( isset( $gi ) ) {
				$gi->geoip_close();
			}
		}
		return $country_code;
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wpadcenter_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
